/*
 * Copyright (c) 2012-present Marvelution Holding B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
	'use strict';

	const dataRestEventHandler = function (event) {
		const $this = $(this);
		if ($this.is(':disabled') || $this.attr('disabled')) {
			return;
		}

		event.preventDefault();

		const that = this;
		if (!isBusy(that)) {
			setBusy(that);

			let checked = that.checked;
			if (typeof(checked) !== 'boolean') {
				checked = checked !== null && checked !== undefined;
			}

			const $that = $(that);
			let options = {
				url: $that.attr('data-rest'),
				type: $that.attr('data-rest-method')
			};
			if (checked !== undefined) {
				options.contentType = 'application/json';
				options.data = JSON.stringify({ 'enabled': checked });
			}

			$.ajax(options).done(function () {
				const message = $that.attr(checked ? 'data-rest-success-checked' : 'data-rest-success');
				showFlag({
					type: 'success',
					title: message || $that.attr('data-rest-success'),
					close: 'auto'
				});
			}).fail(function () {
				if (checked !== undefined) {
					that.checked = !checked;
				}
				const message = $that.attr(checked ? 'data-rest-failed-checked' : 'data-rest-failed');
				showFlag({
					type: 'error',
					title: message || $that.attr('data-rest-failed')
				});
			}).always(function () {
				setIde(that);
			});
		}
	};
	$(document).on('click', '.aui-button[data-rest]', dataRestEventHandler);
	$(document).on('change', 'aui-toggle[data-rest]', dataRestEventHandler);
	$(document).on('click', 'aui-item-checkbox[data-rest]', dataRestEventHandler);
	$(document).on('click', 'aui-item-link[data-rest]', dataRestEventHandler);

	const isBusy = function (element) {
		if (typeof(element.isBusy) === 'function') {
			return element.isBusy();
		} else if (typeof(element.busy) === 'boolean') {
			return element.busy;
		} else {
			return $(element).data('busy');
		}
	};

	const setBusy = function (element) {
		if (typeof(element.busy) === 'function') {
			element.busy();
		} else if (typeof(element.busy) === 'boolean') {
			element.busy = true;
		} else {
			$(element).data('busy', true);
		}
	};

	const setIde = function (element) {
		if (typeof(element.idle) === 'function') {
			element.idle();
		} else if (typeof(element.busy) === 'boolean') {
			element.busy = false;
		} else {
			$(element).data('busy', false);
		}
	};
})(jQuery);
