/*
 * Copyright (c) 2012-present Marvelution Holding B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function ($) {
	'use strict';

	$(document).on('keydown', 'textarea.autosize', function () {
		const element = this;
		setTimeout(function () {
			element.style.height = '0px';
			element.style.height = element.scrollHeight + 'px';
		}, 0);
	});

	const handleError = function (action, xhr) {
		showFlag({
			type: 'error',
			title: I18n.getText(action),
			body: I18n.getTextOrUndefined(action + '.body'),
			close: 'auto'
		});
		if (xhr) {
			let data;
			try {
				data = $.parseJSON(xhr.responseText) || {};
			} catch (e) {
				data = {};
			}
			if (data.errors) {
				$.each(data.errors, function () {
					let $form;
					if (this.context) {
						$form = $('form.' + this.context);
						$form.closest('.configuration').addClass('editing');
					} else {
						$form = $(document);
					}

					let $errorContainer = $('[name=' + this.field + ']:not([disabled])', $form).parent().find('.error');
					if ($errorContainer.length === 0) {
						$errorContainer = $form.find('.fields > .error');
					}
					$errorContainer.closest('.more').addClass('expanded');

					$errorContainer.html(this.message);
				});
			}
		}
	};

	const addHandler = function (button, type) {
		if (!button.isBusy()) {
			button.busy();

			const $select = $('select[name="' + type + '"]');
			$select.parent().find('.error').html('');
			const conditionId = $select.val();

			if (conditionId) {
				$.ajax({
					url: $('.automation-rule>form').attr('action') + '/form/' + type + '/' + conditionId,
					type: 'GET',
					data: {
						index: $select.closest('.rule-element').find('.rule-' + type).length
					}
				}).done(function (html) {
					button.idle();
					$select.closest('.more').toggleClass('expanded').before(html)
							.closest('.rule-element').addClass('configured');
				}).fail(function (xhr) {
					button.idle();
					handleError('rule.' + type + '.load.failed', xhr);
				});
			} else {
				button.idle();
				handleError('rule.' + type + '.not.selected');
			}
		}
	};

	$(document).on('click', '.more .aui-button', function (event) {
		const $button = $(this);

		if ($button.hasClass('add-more') || $button.hasClass('cancel')) {
			event.preventDefault();
			$button.closest('.more').toggleClass('expanded');
		} else if ($button.hasClass('add')) {
			event.preventDefault();
			addHandler(this, $button.attr('data-rule-element-type'));
		}
	});

	$(document).on('click', '#make-conditional', function (event) {
		event.preventDefault();

		$(this).closest('.rule-element').addClass('conditional');
	});

	$(document).on('click', '.edit', function (event) {
		event.preventDefault();

		$(this).closest('.configuration').toggleClass('editing');
	});

	$(document).on('click', '.delete', function (event) {
		event.preventDefault();

		const element = $(this).closest('.rule-element');

		$(this).closest('.configuration').remove();

		if (!element.find('.configuration').length) {
			element.removeClass('configured');
		}
	});

	const loadElement = function (elementType, index) {
		let $form;
		if (index.jquery) {
			$form = index;
		} else {
			$form = $('form.' + elementType + '-' + index);
		}
		const $elementForm = $form.serializeJson();
		let elementData = {configuration: {}};
		elementData[elementType + 'Id'] = $elementForm.elementId;
		delete $elementForm.elementId;
		elementData.configuration = $elementForm;
		return elementData;
	};

	const loadElements = function (elementType) {
		const $elements = $('.rule-' + elementType);
		const count = $elements.length;
		let data = [];
		for (let i = 0; i < count; i++) {
			data.push(loadElement(elementType, $($elements[i]).find('form')));
		}
		return data;
	};

	$(document).on('click', '#save-rule-button', function (event) {
		event.preventDefault();

		const button = this;
		if (!button.isBusy()) {
			button.busy();

			const $form = $('.automation-rule>form');
			let rule = $.extend({}, $form.serializeJson(), {
				when: loadElement('when', 0),
				conditions: loadElements('condition'),
				actions: loadElements('action')
			});
			rule.when.handlerId = rule.when.whenId;
			delete rule.when.whenId;

			$('.error', '.automation-rule').html('');

			$.ajax({
				url: $form.attr('action'),
				type: 'POST',
				data: JSON.stringify(rule),
				contentType: 'application/json',
				dataType: 'text'
			}).done(function (id) {
				showFlag({
					type: 'success',
					title: I18n.getText('save.rule.successful'),
					close: 'auto'
				});
				button.idle();
				$form.find('input[name=id]').val(id);
			}).fail(function (xhr) {
				button.idle();
				handleError('save.rule.failed', xhr)
			});
		}
	});

})(jQuery);
